# go-idx-engine

A go module that provides the GO interface to the _infospace_ and _vank_ libraries that implement _infospace management_ using a _search engine_.

------------------
Build instructions
------------------

Pre-Requisites

1) The vank library should be built first and installed in the prefix directory $MY_INSTALL_HOME specified below.

2) The infospace library should be built and installed in the prefix directory $MY_INSTALL_HOME specified below.

Linux platform

```
cd $PROJECT_ROOT
make -f Makefile.cgo clean
make -f Makefile.icgo clean
make -f Makefile.cgo
make -f Makefile.icgo
go clean
swig -go -cgo -c++ -intgosize 64 vank.i
go build -a -v
go test -v
```

__Notes__
- the swig interface files (*.i) and cgo directives __must__ match the equivalent files in the respective infospace and vank libraries.

- the cgo directives are generated by Makefilie.cgo from vank, and Makefile.icgo from infospace. the swig compile command adds the directives to the top of vank_go.go as __import "C"__.

- when the pimary interface file vank.i is modified, swig compilation may assign go type names a different unique numeric sub-string referring to the wrapped c++ types that cause a subsequent go build to fail. When this happens, interface files that define go types for c++ types need to be modified and re-compiled with swig. the following patterns highlight the substring that should match:

	i) find the types assigned new unique number:

```
egrep --color '_vank_go_[0123456789abcdef]{16}' vank_go.go
extern void _wrap_Swig_free_vank_go_c2859e9b02de2bf9(uintptr_t arg1);
```

	ii) edit the references to those types in referencing interface files:

```
egrep --color '_vank_go_[0123456789abcdef]{16}' *.i
IndexEnvironment_post.i:    return (IndexEnvironment)(SwigcptrWrapped_IndexEnvironment(C._wrap_new_Wrapped_IndexEnvironment_vank_go_8e24520e0b567faa()))
```

	iii) re-compile using the swig command above.

- this module wraps c++ exceptions and converts them into GO errors. the mechanism relies on c++ and CGO implementations and is not guaranteed to always properly intercept the c++ side exception. the mechanism used is described in the two interface files:
	```
	protect_post.i
	protect_pre.i
	```

- the following vank interface files __must__ match vank/swig/src/interface/:
	```
	ConflationPattern.i
	DocIdVector_post.i
	DocumentVector.i
	greedy_vector.i
	IndexEnvironment.i
	IndexEnvironment_post.i
	IntVector.i
	LemurException.i
	MetadataPair.i
	MetadataPair_post.i
	MetadataPairVector.i
	MetadataPairVector_post.i
	MResults.i
	Parameters.i
	Parameters_post.i
	ParsedDocument.i
	QueryAnnotationNode.i
	QueryEnvironment.i
	QueryExpander.i
	QueryRequest.i
	QueryResults.i
	ScoredExtentArray.i
	SnippetBuilder.i
	Specification.i
	Specification_post.i
	StringMap.i
	StringPair.i
	StringPairVector.i
	StringPairVector_post.i
	StringVector.i
	VankBuildIndex.i
	VankBuildIndex_post.i
	vank.i
	vanktypemaps.i
	```
- the following vank interface files parts that reference infospace are commented out in vank/swig/src/interface/:
	```
	DocIdVector.i
	DomainPtrVector.i
	protect_post.i
	protect_pre.i
	```

- the following infospace interface files __must__ match infospace/swig/src/interface/:
	```
	Odms.i
	Odms_post.i
	```
